#' Create a box plot for one or more genes across conditions
#'
#' This function generates a boxplot of fitness coefficients from an object created with the fitCofComp function.
#' The set of genes plotted can is specified as a string (for a single gene) or an array of strings (for multiple genes).
#' Colors can be specified in a similar way (one color per gene in the same order).
#'
#' @param fitCofComp A comparison of fitness coefficients generated by the fitCofComp function.
#' @param gene A list of genes to include in the box plot.
#' @param colors A list of colors to use for the included genes.
#' @param ylim The limits of the y axis.
#' @param las The text orientation parameter.
#' @param dataPoints A single-letter string specifying the character to use to draw the data points
#' 
#' @export
boxplotFCs = function(fitCofComp, gene, colors = NULL, ylim = NULL, las = 2, dataPoints = "") {
    if (is.null(colors)) {
        colors = (2:(length(gene)+1))
    }
    if (length(gene) == 1) {
        boxplot(fitCofComp[grep(gene,row.names(fitCofComp)),] ~ factor(sub("_[1-9]$","",colnames(fitCofComp)), levels = unique(sub("_[1-9]$","",colnames(fitCofComp)))), las = las, cex.lab = 0.8, cex.axis = 0.6, xlab = "", ylab = "Fitness coefficient", range = 0, main = grep(gene,row.names(fitCofComp), value = TRUE), ylim = ylim)

        treatmentN = length(unique(sub("_[1-9]$","",colnames(fitCofComp))))
        nrep = length(colnames(fitCofComp))/treatmentN
        repvec = c()
        for (bpi in 1:treatmentN) {
            repvec = c(repvec,rep(bpi,nrep))
        }
            points(repvec, fitCofComp[grep(gene,row.names(fitCofComp)),], pch = rep(dataPoints,length(colnames(fitCofComp))), cex = 0.5)
        
    } else {
        treatmentNames = unique(sub("_[1-9]$","",colnames(fitCofComp)))
        treatmentN = length(treatmentNames)
        values = fitCofComp[grep(gene[1],row.names(fitCofComp)),]
        nsamp = length(colnames(fitCofComp))
        stratifyBy = sub("_[1-9]$","",colnames(fitCofComp))
        geneNames = rep(gene[1],nsamp)
        for (g in 2:length(gene)) {
            values = c(values,fitCofComp[grep(gene[g],row.names(fitCofComp)),])
            stratifyBy = c(stratifyBy,sub("_[1-9]$","",colnames(fitCofComp)))
            geneNames = c(geneNames,rep(gene[g],nsamp))
        }
        data = data.frame(geneNames, stratifyBy, values)
        stratifyByF = factor(stratifyBy)
        stratifyByF = factor(stratifyByF,levels = treatmentNames)
        geneNamesF = factor(geneNames)
        myplot = boxplot(values ~ geneNamesF * stratifyByF, data = data, las = 2, cex.lab = 0.8, cex.axis = 0.6, boxwex = 0.5, xlab = "", ylab = "Fitness coefficient", range = 0, main = "",col=colors, ylim = ylim, xaxt="n")
        axis(1, at = seq(length(gene)/2+0.5 , (length(gene)*treatmentN+1) , length(gene)), labels = treatmentNames ,  tick=FALSE , cex.axis=0.5, las = las)
        for (i in seq(0.5 , (length(gene)*treatmentN+1) , length(gene))) { 
            abline(v=i,lty=2, col="grey")
        }

        nrep = length(values)/treatmentN
        
        repvec = c()
        for (bpi in 1:(length(values)/nrep)) {
            repvec = c(repvec,rep(bpi,nrep))
        }
        points(repvec, values, pch = rep(dataPoints,length(values)), cex = 0.5)
        
        
        legend("bottomleft", legend = levels(geneNamesF), col=colors, pch = 15, bg = "white", pt.cex = 0.4, cex = 0.5,  horiz = T, inset = c(0, 0))
    }
}
