#' Performs a gene set analysis
#'
#' Performs a gene set analysis based on a supplied GSC (see generateGSC function) and a data set produced by the rankByP function.
#' By default, only genes with a significantly different fitness coefficient (p < 0.05) are included in the gene set analysis. This can be changed by changing the significance cutoff to any value between 0 and 1.
#' On each gene set, a statistical test of differential sum of fitness coefficients is performed.
#' Be default, this is a t-test, but this can be changed to either “t.test” or “wilcox” (for the Wilcoxon signed rank test).
#' If center is set to TRUE, the mean fitness coefficient of each sample is subtracted from that sample before gene set analysis is performed. By default, this centration of data is not performed.
#'
#' @param GSC Gene Set Collection object fenerateed by the generateGSC function.
#' @param dataset A data set generated by the rankByP function.
#' @param p Cutoff p-value for inclusion in the gene set analysis.
#' @param test Type of statistical test to use (default is "t.test", but can also be "wilcox").
#' @param center If TRUE, the mean fitness coefficient of each sample is subtracted from all values in that sample before analysis.
#' @return Returns a table of gene sets (e.g. pathways) and the number of genes in each set with a positive fitness coefficient, a negative fitness coefficient, the number of non-significant genes, the estimated average fitness coefficient in the set, the uncorrected p-value for the enrichment, and the FDR corrected p-value for enrichment.
#'
#' @export
genesetAnalysis = function(GSC, dataset, p = 0.05, test = "t.test", center = FALSE) {
  if (center == TRUE) {
    moddataset = dataset
    moddataset[,1] = dataset[,1] - mean(dataset[,1])
  } else {
    moddataset = dataset
  }
  row.names(moddataset) = sub(":.*","",row.names(moddataset))
  hist(moddataset[,1])
  pathways = names(GSC)
  returnList = matrix(0,length(pathways),6)
  row.names(returnList) = unlist(pathways)
  colnames(returnList) = c("Up","Down","Non-significant","Mean","p-value","adj.p")
  for (pn in 1:(length(pathways))) {
    pathwayname = pathways[pn]
    genelist = as.vector(unlist((GSC[[pathwayname]])))
    if (length(genelist) < 2) {
      next()
    }
    pathwaygenes = moddataset[genelist,]
    signGenes = pathwaygenes[pathwaygenes[,3] <= p,]
    if (length(row.names(signGenes)) < 1) {
      genesUp = 0
      genesDown = 0
      genesNS = length(genelist)
      pval = NA
      estval = NA
    } else {
      genesUp = sum(signGenes[,1] > 0)
      genesDown = sum(signGenes[,1] < 0)
      genesNS = length(genelist) - genesUp - genesDown
      if (test == "t.test") {
        testResult = try(t.test(signGenes[,1]), silent = TRUE)
        if (class(testResult) == "try-error") {
            pval = NA
            estval = NA
        } else {
            pval = testResult$p.value
            estval = testResult$estimate
        }
      }
      if (test == "wilcox") {
        testResult = wilcox.test(signGenes[,1])
        pval = testResult$p.value
        estval = abs(genesUp - genesDown)
      }
    }
    returnList[pn,] = c(genesUp,genesDown,genesNS,estval,pval,pval)
  }
  returnList = returnList[returnList[,1] + returnList[,2] + returnList[,3] > 0,]
  returnList[,6] = p.adjust(returnList[,5], method = 'BH')
  return(returnList)
}
